( cd usr/lib ; rm -rf libgdkcardimage.so )
( cd usr/lib ; ln -sf libgdkcardimage.so.0.0.0 libgdkcardimage.so )
( cd usr/lib ; rm -rf libgdkcardimage.so.0 )
( cd usr/lib ; ln -sf libgdkcardimage.so.0.0.0 libgdkcardimage.so.0 )
if [ -d var/lib/games/tmp ]; then
  ( cd var/lib/games/tmp
    for file in * ; do
      if [ ! -r ../$file ]; then
        mv $file ..
      fi
    done )
  rm -rf var/lib/games/tmp
fi
if [ -x /usr/bin/scrollkeeper-update ]; then
  /usr/bin/scrollkeeper-update -p /var/lib/scrollkeeper 1> /dev/null 2> /dev/null
fi
